"""Creates a mailing-list class for interfacing with Qualtrics API v3

This module creates a class for encapsulating a Qualtrics Mailing List based
upon a Qualtrics Account object, Qualtrics Library/Group id, and settings for
mailing-list name and category name

"""

import json

import pandas as pd
import requests
from .qualtrics_account import QualtricsAccount


class QualtricsMailingList(object):
    """Mailing-list Class for interfacing with Qualtrics API v3"""
    def __init__(
            self,
            account: QualtricsAccount,
            library_id: str,
            mailing_list_name: str,
            category_name: str
    ):
        """Initializes a Qualtrics mailing-list object

        Args:
            account: a QualtricsAccount object
            library_id: a Qualtrics Library/Group id, which acts similarly to a
                "working directory"; see https://api.qualtrics.com/docs/
                finding-qualtrics-ids
            mailing_list_name: the name for the mailing list being created
            category_name: the category name for the mailing list being created,
                which acts similarly to a "tag" for the mailing list
        """
        self.account = account
        self.library_id = library_id
        self.mailing_list_name = mailing_list_name
        self.category_name = category_name

        # make Qualtrics API v3 call to create mailing list
        request_response = requests.request(
            "POST",
            f"https://{self.account.data_center}.qualtrics.com"
            f"/API/v3/mailinglists/",
            data=(
                f'{{"libraryId": "{library_id}",'
                f'"name": "{mailing_list_name}",'
                f'"category": "{category_name}" }}'
            ),
            headers={
                "content-type": "application/json",
                "x-api-token": self.account.api_token,
            },
        )

        # extract mailing list id from HTTP response
        self.id = request_response.json()["result"]["id"]

    def import_contact_list_from_csv_file(self, fp) -> None:
        """Imports a contact list from a csv file using a Qualtrics v3 API call.

        Args:
            fp: pointer to csv file or file-like object having a non-empty
                subset of the following column headers in the next line to be
                read and corresponding data in all following lines:
                - id
                - firstName
                - lastName
                - email
                - language
                - unsubscribed
                - externalReference
                see https://api.qualtrics.com/docs/create-contacts-import

        """

        # convert csv file contents to JSON records format
        contact_list = json.loads(pd.read_csv(fp).to_json(orient='records'))

        # make Qualtrics API v3 call to upload contact list
        request_response = requests.request(
            "POST",
            f"https://{self.account.data_center}.qualtrics.com"
            f"/API/v3/mailinglists/{self.id}/contactimports",
            json={"contacts": contact_list},
            headers={
                "content-type": "application/json",
                "x-api-token": self.account.api_token,
            },
        )

        # check upload progress until complete
        progress_id = request_response.json()["result"]["id"]
        request_check_progress = 0
        while request_check_progress < 100:
            request_response = requests.request(
                "GET",
                f"https://{self.account.data_center}.qualtrics.com"
                f"/API/v3/mailinglists/{self.id}/contactimports/{progress_id}",
                headers={
                    "x-api-token": self.account.api_token,
                },
            )
            request_check_progress = request_response.json()["result"][
                "percentComplete"]

    @property
    def contact_list(self) -> dict:
        """Returns mailing list's contact list without caching"""
        request_response = requests.request(
            "GET",
            f"https://{self.account.data_center}.qualtrics.com"
            f"/API/v3/mailinglists/{self.id}/contacts",
            headers={
                "x-api-token": self.account.api_token,
            },
        )
        return request_response.json()["result"]["elements"]



"""Creates an account class for interfacing with Qualtrics API v3

This module creates a class for encapsulating a Qualtrics Account based upon API
token and data center, including a method for reading an API token from a file.

"""


class QualtricsAccount(object):
    """Account Class for interfacing with Qualtrics API v3"""

    def __init__(
            self,
            api_token: str = None,
            data_center: str = None
    ):
        """Initializes QualtricsAccount object using API Token and Date Center

        Args:
            api_token: a Qualtrics API v3 token; see https://www.qualtrics.com/
                support/integrations/api-integration/overview/
            data_center: a Qualtrics account data center specific to the account
                associated with the API token; see https://api.qualtrics.com/
                docs/root-url

        """
        self.api_token = api_token
        self.data_center = data_center

    def set_api_token_from_file(self, fp) -> None:
        """Loads Qualtrics API v3 token from file

        Args:
            fp: pointer to file or file-like object that is ready to read from
                and contains a Qualtrics API v3 token as the next line to read

        """
        self.api_token = fp.readline()



"""Creates a survey-distribution class for interfacing with Qualtrics API v3

This module creates a class for encapsulating a Qualtrics Survey Distribution
based upon a Qualtrics Mailing List object, Qualtrics Message id, Qualtrics
Survey id, and email settings for send date, from name, reply-to email address,
subject, and from email address, which defaults to "noreply@qemailserver.com"

"""


import requests

from .qualtrics_mailing_list import QualtricsMailingList


class QualtricsDistribution(object):
    """Survey-Distribution Class for interfacing with Qualtrics API v3"""
    def __init__(
        self,
        mailing_list: QualtricsMailingList,
        message_id: str,
        survey_id: str,
        send_date: str,
        from_name: str,
        reply_email: str,
        subject: str,
        *,
        from_email: str = "noreply@qemailserver.com",
    ):
        """Initializes a Qualtrics survey-distribution object

        Args:
            mailing_list: a QualtricsMailingList object with initialized
                Qualtrics Account sub-object
            message_id: a Qualtrics message id; see https://api.qualtrics.com/
                docs/finding-qualtrics-ids
            survey_id: a Qualtrics survey id; see https://api.qualtrics.com/
                docs/finding-qualtrics-ids
            send_date: the send datetime for the survey distribution being
                created in ISO 8601 format; see https://api.qualtrics.com/
                docs/dates-and-times
            from_name: the from name for the survey distribution being created
            reply_email: the reply-to email address for the survey distribution
                being created
            subject: the email subject for the survey distribution being created
            from_email: the from name for the survey distribution being created,
                defaulting to the Qualtrics-supplied noreply@qemailserver.com;
                see https://www.qualtrics.com/support/survey-platform/
                distributions-module/email-distribution/emails/
                using-a-custom-from-address/

        """
        self.mailing_list = mailing_list
        self.message_id = message_id
        self.survey_id = survey_id
        self.send_date = send_date
        self.from_name = from_name
        self.reply_email = reply_email
        self.subject = subject
        self.from_email = from_email

        # make Qualtrics API v3 call to create survey distribution
        request_response = requests.request(
            "POST",
            f"https://{self.mailing_list.account.data_center}.qualtrics.com"
            f"/API/v3/distributions/",
            headers={
                "x-api-token": self.mailing_list.account.api_token,
                "Content-Type": "application/json",
            },
            json={
                "surveyLink": {
                    "surveyId": self.survey_id,
                },
                "header": {
                    "fromEmail": self.from_email,
                    "fromName": self.from_name,
                    "replyToEmail": self.reply_email,
                    "subject": self.subject,
                },
                "message": {
                    "libraryId": self.mailing_list.library_id,
                    "messageId": self.message_id,
                },
                "recipients": {
                    "mailingListId": self.mailing_list.id,
                },
                "sendDate": self.send_date,
            }
        )

        # extract distribution id from HTTP response
        self.id = request_response.json()["result"]["id"]

    @property
    def details(self) -> dict:
        """Returns survey-distribution details without caching"""
        request_response = requests.request(
            "GET",
            f"https://{self.mailing_list.account.data_center}.qualtrics.com"
            f"/API/v3/distributions/{self.id}?surveyId={self.survey_id}",
            headers={
                "x-api-token": self.mailing_list.account.api_token
            },
        )
        return request_response.json()['result']



from setuptools import setup

setup(
    name='qualtrics_mailer',
    version='0.1',
    description='A package for distributing pre-built surveys in Qualtrics',
    keywords='qualtrics survey',
    url='http://github.com/kaianalytics/qualtrics_mailer',
    author='Kai Analytics',
    author_email='info@kaianalytics.com',
    license='MIT',
    classifiers=[
        'Programming Language :: Python :: 3.6'
    ],
    packages=[
        'qualtrics_mailer'
    ],
    install_requires=[
        'pandas',
        'requests'
    ],
    zip_safe=False
)
